<?php
/**
 * Plugin Name: Remote control panel
 *
 * Description: Plugin to enable remote of wordpress via wpremotecontrolpanel.com
 *
 * Plugin URI: http://wordpress.org/extend/plugins/remotecontrolpanel/
 * Version: 1.0
 * Author: Erik Torsner
 * Author URI: http://erik.torgesta.com
 * @package remotecontrol
 * @copyright Torgesta Technology AB 2013
 */

require_once 'lib/remotecontrolpanel_utils.php';

/**
 * The instantiated version of this plugin's class
 */
$GLOBALS['remotecontrolpanel'] = new remotecontrolpanel();

/**
 * Main class
 *
 * @package remotecontrolpanel
 * @link http://wordpress.org/extend/plugins/remotecontrolpanel/
 * @author Erik Torsner <erik@torgesta.com>
 * @copyright Torgesta Technology AB 2013
 *
 */
class remotecontrolpanel {
	/**
	 * This plugin's identifier
	 */
	const ID = 'remotecontrolpanel';

	/**
	 * This plugin's name
	 */
	const NAME = 'Remote Control Panel';

	/**
	 * This plugin's version
	 */
	const VERSION = '1.0';

	/**
	 * This plugin's options
	 *
	 */
	protected $options = array();

	/**
	 * This plugin's default options
	 * @var array
	 */
	protected $options_default = array(
		'apikey'       => '[notset]',
	);

	/**
	 * Our option name for storing the plugin's settings
	 * @var string
	 */
	protected $option_name;

	/**
	 * Declares the WordPress action and filter callbacks
	 *
	 * @return void
	 * @uses oop_plugin_template_solution::initialize()  to set the object's
	 *       properties
	 */
	public function __construct() 
	{
		$this->initialize();
		$this->utils = new remotecontrolpanel_utils();
		$this->snapshotsfolder =  WP_CONTENT_DIR . '/rcpsnapshots';

		if (is_admin()) 
		{
			require_once dirname(__FILE__) . '/admin.php';
			$admin = new remotecontrolpanel_admin;
			$info = pathinfo(__FILE__);
			$filter = $info['filename'] . '/' . $info['basename'];

			if (is_multisite()) 
			{
				$admin_menu = 'network_admin_menu';
				$admin_notices = 'network_admin_notices';
				$plugin_action_links = 'network_admin_plugin_action_links_' . $filter;
			} 
			else 
			{
				$admin_menu = 'admin_menu';
				$admin_notices = 'admin_notices';
				$plugin_action_links = 'plugin_action_links_' . $filter;
			}

			add_action($admin_menu, array(&$admin, 'admin_menu'));
			add_action('admin_init', array(&$admin, 'admin_init'));
			add_filter($plugin_action_links, array(&$admin, 'plugin_action_links'));

			register_activation_hook(__FILE__, array(&$admin, 'activate'));
			register_deactivation_hook(__FILE__, array(&$admin, 'deactivate'));

		}
		else
		{
			add_action( 'init', array(&$this, 'detect_api_request'), 1);
		}
	}

	/**
	 * Sets the object's properties and options
	 *
	 * This is separated out from the constructor to avoid undesirable
	 * recursion.  The constructor sometimes instantiates the admin class,
	 * which is a child of this class.  So this method permits both the
	 * parent and child classes access to the settings and properties.
	 *
	 * @return void
	 *
	 * @uses oop_plugin_template_solution::set_options()  to replace the default
	 *       options with those stored in the database
	 */
	protected function initialize() {
		$this->option_name = self::ID . '-options';
		$this->set_options();

		//add_filter('query', array($this, 'rcp_handle_query'), 99, 1);
	}

	public function rcp_handle_query($query) {
		$fp = fopen($this->snapshotsfolder . '/queries', 'a');
		fwrite($fp, $query . "\n");
		fclose($fp);

		return $query;
	}


	/**
	 * Replaces the default option values with those stored in the database
	 *
	 * @return  void
	 */
	protected function set_options() {
		if (is_multisite()) 
		{
			switch_to_blog(1);
			$options = get_option($this->option_name);
			restore_current_blog();
		} 
		else 
		{
			$options = get_option($this->option_name);
		}
		if (!is_array($options)) 
		{
			$options = array();
		}

		$this->options = array_merge($this->options_default, $options);
	}

	/**
	 * Determine early if the current http request is for
	 * RCP to handle. If so, RCP will to it's job and exit
	 * out so that no other part of WP tries to deal with it.
	 * 
	 * @return void
	 */
	public function detect_api_request()
	{
		// Is a token set
		if( ! isset($_REQUEST['ttech_rcp_token'])) return FALSE;
		$token = $_REQUEST['ttech_rcp_token'];
		// Are there any actions?
		if( ! isset($_REQUEST['ttech_rcp_action'])) return FALSE;
		$action = $_REQUEST['ttech_rcp_action'];

		// Handle the API request, and then exit.
		$this->handle_api_request($action, $token);
		exit;
	}

	/**
	 * Check if a provided token is valid
	 * 
	 * @param  string $token Token passed via http
	 * 
	 * @return bool TRUE/FALSE
	 */
	private function validate_token($token)
	{
		$retval = FALSE;
		$apikey = $this->options['apikey'];
		$salt   = $_SESSION['salt'];
		if($token == sha1($apikey . $salt))
		{
			$retval =  TRUE;
		}
		return $retval;
	}

	/**
	 * Do the actual work of a RCP request
	 * 
	 * @param  string $action Command
	 * @param  string $token  Token
	 * 
	 * @return void Return is written as JSON on std out
	 */
	private function handle_api_request($action, $token)
	{
		$ret = new stdClass();
		$ret->mem_start = memory_get_usage(TRUE);
		if(!session_id()) {
		        session_start();
			if(!isset($_SESSION['salt']))
				$_SESSION['salt'] = sha1(microtime(TRUE) . mt_rand() . 'salt');
		}

		$valid_token = $this->validate_token($token);
		$non_auth_actions = array('init');

		if(!in_array($action, $non_auth_actions))
		{
			if(!$valid_token)
			{
				$ret->action = $action;
				$ret->status = 502;
				$ret->message = 'Invalid api key or out of squence';
				$action = 'invalid_token';
			}
		}
		$this->impersonate();
		switch($action)
		{
			case 'init':
			{
				$ret->action = $action;
				$ret->status  = 200;
				break;
			}
			case 'noop':
			{
				// used to check that the API key is OK. If we're here
				// it means it was.
				$ret->status = 200;
				break;
			}
			case 'get_version':
			{
				$ret->action = $action;
				$ret->status  = 200;
				$ret->version   = self::VERSION;
				break;
			}
			case 'get_site_info':
			{
				global $wp_version;
				$ret->action       = $action;
				$ret->status       = 200;
				$ret->site_url     = get_site_url();
				$ret->home_url     = get_home_url();
				$ret->admin_url    = get_admin_url();
				$ret->abspath    = ABSPATH;
				$ret->wp_version   = (string) $wp_version;
				$ret->capabilities = $this->utils->get_capabilities();
				break;
			}				
			case 'get_plugins':
			{	
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();
				$ret->action  = $action;

				$ret->status  = 200;
				$ret->plugins = $plugins->get_plugins();

				break;
			}
			case 'upgrade_plugin':
			{	
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();

				$pluginname = $_REQUEST['plugin'];

				$ret->action  = $action;
				$ret->plugin = $pluginname;
				$retArr = $plugins->upgrade_plugin($pluginname);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
					$ret->reactivate = $retArr['reactivate'];
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}
			case 'install_plugin':
			{
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();
				$url = $_POST['url'];

				$ret->action  = $action;
				$retArr = $plugins->install_plugin($url);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
					$ret->plugin_file = $retArr['plugin_file'];
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;				

			}
			case 'activate_plugin':
			{
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();
				$pluginname = $_REQUEST['plugin'];

				$ret->action  = $action;
				$ret->plugin = $pluginname;
				$retArr = $plugins->activate_plugin($pluginname);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}
			case 'run_activate_plugin':
			{
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();

				$pluginname = $_REQUEST['plugin'];

				$ret->action  = $action;
				$ret->plugin = $pluginname;
				$retArr = $plugins->run_activate_plugin($pluginname);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}
			case 'deactivate_plugin':
			{
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();

				$pluginname = $_REQUEST['plugin'];

				$ret->action  = $action;
				$ret->plugin = $pluginname;
				$retArr = $plugins->deactivate_plugin($pluginname);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}			
			case 'delete_plugin':
			{
				require_once('remotecontrolpanel_plugins.php');
				$plugins = new remotecontrolpanel_plugins();

				$pluginname = $_REQUEST['plugin'];

				$ret->action  = $action;
				$ret->plugin  = $pluginname;
				$retArr = $plugins->delete_plugin($pluginname);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}			


			case 'get_themes':
			{	
				require_once('remotecontrolpanel_themes.php');
				$themes = new remotecontrolpanel_themes();

				$ret->action  = $action;

				$ret->status  = 200;
				$ret->themes = $themes->get_themes();

				break;
			}
			case 'upgrade_theme':
			{	
				require_once('remotecontrolpanel_themes.php');
				$themes = new remotecontrolpanel_themes();

				$themename = $_REQUEST['theme'];

				$ret->action  = $action;
				$ret->theme = $themename;
				$retArr = $themes->upgrade_theme($themename);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}
				break;
			}
			case 'get_core':
			{	
				require_once('remotecontrolpanel_core.php');
				$core = new remotecontrolpanel_core();

				$ret->action  = $action;

				$ret->status  = 200;
				$ret->core = $core->get_core();

				break;
			}
			case 'upgrade_core':
			{	
				require_once('remotecontrolpanel_core.php');
				$core = new remotecontrolpanel_core();

				$ret->action  = $action;
				$retArr = $core->upgrade_core($themename);
				if($retArr['status'] == 200)
				{
					$ret->status  = 200;
				}
				else
				{
					$ret->status  = 500;
					$ret->error = $retArr['error'];
				}				
				break;
			}
			case 'set_storage_credentials':
			{
				$this->set_storage_credentials();
				$ret->action = $action;
				$ret->status = 200;
			}
			case 'get_files_state':
			{
				$filename = md5(time());
				umask(007);
				@mkdir($this->snapshotsfolder);
				$path = $this->snapshotsfolder . "/" . $filename;
				$f = fopen($path, 'w');
				remotecontrolpanel_utils::rec_scandir(ABSPATH, $f);
				fclose($f);

				$ret->path = content_url() . '/rcpsnapshots/' . $filename;
				$ret->ref = $filename;
				$ret->action = $action;
				$ret->status  = 200;
				break;				
			}
			case 'get_db_state':
			{
				$filename = md5(time());
				umask(007);
				@mkdir($this->snapshotsfolder);
				$path = $this->snapshotsfolder . "/" . $filename;
				$f = fopen($path, 'w');
				remotecontrolpanel_utils::get_db_state($f);
				fclose($f);

				$ret->path = content_url() . '/rcpsnapshots/' . $filename;
				$ret->ref = $filename;
				$ret->action = $action;
				$ret->status  = 200;
				break;	
			}

			case 'clear_statefile':
			{
				$path = $this->snapshotsfolder . "/" . $_REQUEST['fileref'];
				if(file_exists($path)) unlink($path);
				$ret->fileref = $path;
				$ret->action = $action;
				$ret->status  = 200;
				break;
			}
			case 'get_file':
			{
				$fileref = $_REQUEST['fileref'];
				$content = file_get_contents(ABSPATH . $fileref);
				$ret->action = $action;                
				if($content)
				{
					$ret->data = $content;
					$ret->status  = 200;

				} else {
					$ret->status = 500;
				}
				break;
			}
			case 'prepare_files':
			{
				$postdata = file_get_contents("php://input");
				$files = json_decode($postdata);
				$filename = sha1($postdata . microtime(TRUE) . mt_rand());
				$path = $this->snapshotsfolder . "/$filename";

				$result = remotecontrolpanel_utils::compress_filelist($path, $files);
				$ret->action = $action;
				if($result)
				{
					$ret->ref = $filename;					
					$ret->path = content_url() . '/rcpsnapshots/' . $filename;
					$ret->status  = 200;

				} else {
					$ret->status = 500;
				}
				break;
			}

			case 'prepare_tables':
			{
				$postdata = file_get_contents("php://input");
				$tables = json_decode($postdata);
				$filename = sha1(microtime(TRUE) . mt_rand());
				$path = $this->snapshotsfolder . "/$filename";
				$result = remotecontrolpanel_utils::compress_tables(
					$path, $tables, $this->snapshotsfolder);
				if($result)
				{
					$ret->ref = $filename;					
					$ret->path = content_url() . '/rcpsnapshots/' . $filename;
					$ret->status  = 200;
				} else {
					$ret->status = 500;
				}
			}
			break;

			case 'invalid_token':
			{
				break;
			}
			default:
			{
				$ret->status  = 500;
				$ret->error = 'Invalid command';
			}
		}

		// generate the next dynamic salt

		$_SESSION['salt'] = sha1($_SESSION['salt'] . rand() . 'nextToken');
		$ret->nextsalt = $_SESSION['salt'];
		$ret->mem_end = memory_get_usage(TRUE);
		$ret->mem_peak = memory_get_peak_usage(TRUE);

		echo json_encode($ret);
		return;			
	}

	public function set_storage_credentials()
	{
		$cred = new stdClass();
		if(isset($this->options['credentials'])) $cred = $this->options['credentials'];

		$raw_cred = file_get_contents('php://input');
		$new_cred = json_decode($raw_cred);
		switch($new_cred->name)
		{
			case 'dropbox':
			{
				$cred->dropbox = new stdClass();
				$cred->dropbox->consumer_key    = $new_cred->consumer_key;
				$cred->dropbox->consumer_secret = $new_cred->consumer_secret;
				$cred->dropbox->oauth_key       = $new_cred->oauth_key;
				$cred->dropbox->oauth_secret    = $new_cred->oauth_secret;
				$this->options['cred'] = $cred;
				update_option($this->option_name, $this->options);
				break;
			}
		}
	}


	private function impersonate()
	{
		global $current_user;
		$current_user = new WP_User( 1, 'RemoteControlPanel' );
		$current_user->add_cap('activate_plugins');
		$current_user->add_cap('install_themes');
		$current_user->add_cap('install_plugins');
		$current_user->add_cap('update_plugins');
		$current_user->add_cap('update_themes');
		$current_user->add_cap('update_core');
		$current_user->add_cap('delete_plugins');
		$current_user->add_cap('delete_themes');
	}
}
